@@ -100,6 +100,77 @@ CREATE TABLE alerts_trash
   creation_time date,
   modification_time date);
 
+--
+-- Credentials.
+--
+-- This table stores the credentials for LSCs and other functions that require
+--  authentication.
+-- In addition the names for packages or tools to be installed on the
+--  targets are stored.
+CREATE TABLE credentials
+ (id SERIAL PRIMARY KEY,
+    uuid text UNIQUE NOT NULL,
+  owner integer REFERENCES users (id) ON DELETE RESTRICT,
+    -- The OpenVAS name for this credential.
+  name text NOT NULL,
+  comment text,
+  creation_time date,
+  modification_time date,
+    -- The type abbreviation of the credential,
+    --  e.g. "up" for "username + password"
+  type text,
+  allow_insecure integer);
+
+--
+-- Trashcan for credentials.
+--
+CREATE TABLE credentials_trash
+ (id SERIAL PRIMARY KEY,
+  uuid text UNIQUE NOT NULL,
+  owner integer REFERENCES users (id) ON DELETE RESTRICT,
+  name text NOT NULL,
+  comment text,
+  creation_time date,
+  modification_time date,
+  type text,
+  allow_insecure integer);
+
+--
+-- Data table for credentials
+--
+CREATE TABLE credentials_data
+ (id SERIAL PRIMARY KEY,
+    -- The credential the data entry belongs to.
+  credential integer REFERENCES credentials (id),
+    -- The type of data for unencrypted data like "username"
+    --  or "secret" for a container of encrypted data (see "value" column).
+  type TEXT,
+    -- The actual data or an encrypted data container.
+    -- The format of this container is a list of name-value pairs.
+    --  For example:
+    --   <len(8)> "password" <len> <value>
+    --   <len(10)> "private_key" <len> <value>
+    --  The len fields are encoded as 32 bit big endian unsigned
+    --  integer values.  A value of 0 for the value length is is
+    --  allowed and indicates and empty string.  A missing name
+    --  (e.g. "password", indicates a NULL value for that name.
+    --  This format closely resembles an unencrypted
+    --  credential.  However, recursive encryption is not
+    --  supported.  The container itself is OpenPGP encrypted
+    --  with the binary OpenPGP format wrapped into a standard
+    --  base64 encoding without linefeeds or checksums (i.e. it
+    --  is not the armor format).
+  value TEXT);
+
+--
+-- Trashcan for credentials_data
+--
+CREATE TABLE credentials_trash_data
+ (id SERIAL PRIMARY KEY,
+  credential INTEGER,
+  type TEXT,
+  value TEXT);
+
 CREATE TABLE filters
  (id SERIAL PRIMARY KEY,
   uuid text UNIQUE NOT NULL,
@@ -257,14 +328,10 @@ CREATE TABLE targets
   reverse_lookup_only integer,
   reverse_lookup_unify integer,
   comment text,
-  lsc_credential integer REFERENCES lsc_credentials (id) ON DELETE RESTRICT, -- SSH
-  ssh_port text,
-  smb_lsc_credential integer REFERENCES lsc_credentials (id) ON DELETE RESTRICT,
-  port_range integer REFERENCES port_lists (id) ON DELETE RESTRICT,
+  port_list integer REFERENCES port_lists (id) ON DELETE RESTRICT,
   alive_test integer,
-  creation_time date,
-  modification_time date,
-  esxi_lsc_credential integer REFERENCES lsc_credentials (id) ON DELETE RESTRICT);
+  creation_time integer,
+  modification_time integer);
 
 CREATE TABLE targets_trash
  (id SERIAL PRIMARY KEY,
@@ -276,17 +343,26 @@ CREATE TABLE targets_trash
   reverse_lookup_only integer,
   reverse_lookup_unify integer,
   comment text,
-  lsc_credential integer REFERENCES lsc_credentials (id) ON DELETE RESTRICT, -- SSH
-  ssh_port text,
-  smb_lsc_credential integer REFERENCES lsc_credentials (id) ON DELETE RESTRICT,
-  port_range integer REFERENCES port_lists (id) ON DELETE RESTRICT,
-  ssh_location integer,
-  smb_location integer,
+  port_list integer REFERENCES port_lists (id) ON DELETE RESTRICT,
   port_list_location integer,
-  creation_time date,
-  modification_time date,
-  esxi_lsc_credential integer REFERENCES lsc_credentials (id) ON DELETE RESTRICT),
-  esxi_location integer;
+  alive_test integer,
+  creation_time integer,
+  modification_time integer);
+
+CREATE TABLE targets_login_data
+ (id SERIAL PRIMARY KEY,
+  target integer REFERENCES targets (id) ON DELETE RESTRICT,
+  type text,
+  credential integer REFERENCES credentials (id) ON DELETE RESTRICT,
+  port integer);
+
+CREATE TABLE targets_trash_login_data
+ (id SERIAL PRIMARY KEY,
+  target integer REFERENCES targets_trash (id) ON DELETE RESTRICT,
+  type text,
+  credential integer REFERENCES credentials (id) ON DELETE RESTRICT,
+  port integer,
+  credential_location);
 
 CREATE TABLE configs
  (id SERIAL PRIMARY KEY,
@@ -300,6 +376,7 @@ CREATE TABLE configs
   families_growing integer,
   nvts_growing integer,
   type integer,
+  scanner integer REFERENCES scanners (id) ON DELETE RESTRICT,
   creation_time date,
   modification_time date);
 
@@ -315,6 +392,7 @@ CREATE TABLE configs_trash
   families_growing integer,
   nvts_growing integer,
   type integer,
+  scanner integer REFERENCES scanners (id) ON DELETE RESTRICT,
   creation_time date,
   modification_time date);
 
@@ -323,14 +401,18 @@ CREATE TABLE config_preferences
   config integer REFERENCES configs (id) ON DELETE RESTRICT,
   type text, -- openvasrc section name or NULL for top-level prefs
   name text,
-  value text);
+  value text,
+  default_value text,
+  hr_name text);
 
 CREATE TABLE config_preferences_trash
  (id SERIAL PRIMARY KEY,
   config integer REFERENCES configs_trash (id) ON DELETE RESTRICT,
   type text, -- openvasrc section name or NULL for top-level prefs
   name text,
-  value text);
+  value text,
+  default_value text,
+  hr_name text);
 
 CREATE TABLE scanners
  (id SERIAL PRIMARY KEY,
@@ -342,8 +424,7 @@ CREATE TABLE scanners
   port integer,
   type integer,
   ca_pub text,
-  key_pub text,
-  key_priv text,
+  credential integer REFERENCES credentials (id) ON DELETE RESTRICT,
   creation_time date,
   modification_time date);
 
@@ -357,8 +438,8 @@ CREATE TABLE scanners_trash
   port integer,
   type integer,
   ca_pub text,
-  key_pub text,
-  key_priv text,
+  credential integer,
+  credential_location integer,
   creation_time date,
   modification_time date);
 
@@ -378,12 +459,11 @@ CREATE TABLE tasks
   schedule integer REFERENCES schedules (id) ON DELETE RESTRICT,
   schedule_periods integer,
   schedule_next_time date,
-  slave integer REFERENCES slaves (id) ON DELETE RESTRICT,
   scanner integer REFERENCES scanners (id) ON DELETE RESTRICT,
   config_location integer,
   target_location integer,
   schedule_location integer,
-  slave_location integer,
+  scanner_location integer,
   upload_result_count integer,
   hosts_ordering text,
   alterable integer,
@@ -443,7 +523,8 @@ CREATE TABLE reports
   slave_name text,
   slave_host text,
   slave_port integer,
-  source_iface text);
+  source_iface text,
+  flags integer);
 
 CREATE TABLE report_counts
  (id SERIAL PRIMARY KEY,
@@ -452,7 +533,8 @@ CREATE TABLE report_counts
   severity decimal,
   count integer,
   override integer,
-  end_time integer);
+  end_time integer,
+  min_qod integer);
 
 CREATE TABLE report_format_params
  (id SERIAL PRIMARY KEY,
@@ -537,6 +619,73 @@ CREATE TABLE report_host_details
   name text,
   value text);
 
+CREATE TABLE hosts
+ (id SERIAL PRIMARY KEY,
+  uuid text UNIQUE NOT NULL,
+  owner integer REFERENCES users (id) ON DELETE RESTRICT,
+  name text NOT NULL,
+  comment text,
+  creation_time date,
+  modification_time date);
+
+CREATE TABLE oss
+ (id SERIAL PRIMARY KEY,
+  uuid text UNIQUE NOT NULL,
+  owner integer REFERENCES users (id) ON DELETE RESTRICT,
+  name text NOT NULL,
+  comment text,
+  creation_time date,
+  modification_time date);
+
+CREATE TABLE host_identifiers
+ (id SERIAL PRIMARY KEY,
+  uuid text UNIQUE NOT NULL,
+  host integer REFERENCES hosts (id) ON DELETE RESTRICT,
+  owner integer REFERENCES users (id) ON DELETE RESTRICT,
+  name text NOT NULL,
+  comment text,
+  value text NOT NULL,
+  source_type text NOT NULL,
+  source_id text NOT NULL,
+  source_data text NOT NULL,
+  creation_time date,
+  modification_time date);
+
+CREATE TABLE host_oss
+ (id SERIAL PRIMARY KEY,
+  uuid text UNIQUE NOT NULL,
+  host integer REFERENCES hosts (id) ON DELETE RESTRICT,
+  owner integer REFERENCES users (id) ON DELETE RESTRICT,
+  name text NOT NULL,
+  comment text,
+  os integer REFERENCES oss (id) ON DELETE RESTRICT,
+  source_type text NOT NULL,
+  source_id text NOT NULL,
+  source_data text NOT NULL,
+  creation_time date,
+  modification_time date);
+
+CREATE TABLE host_max_severities
+ (id SERIAL PRIMARY KEY,
+  host integer REFERENCES hosts (id) ON DELETE RESTRICT,
+  severity real,
+  source_type text NOT NULL,
+  source_id text NOT NULL,
+  creation_time date);
+
+CREATE TABLE host_details
+ (id SERIAL PRIMARY KEY,
+  host integer REFERENCES hosts (id) ON DELETE RESTRICT,
+  -- The report that the host detail came from.
+  source_type text NOT NULL,
+  source_id text NOT NULL,
+  -- The original source of the host detail, from the scanner.
+  detail_source_type text,
+  detail_source_name text,
+  detail_source_description text,
+  name text,
+  value text);
+
 CREATE TABLE nvt_preferences
  (id SERIAL PRIMARY KEY,
   name text UNIQUE NOT NULL,
@@ -570,68 +719,6 @@ CREATE TABLE nvt_cves
   oid text,
   cve_name text);
 
---
--- Local Security Check Credentials.
---
--- This table stores the credentials for LSCs.  In addition the names
--- for packages or tools to be installed on the targets are stored.
---
-CREATE TABLE lsc_credentials
- (id SERIAL PRIMARY KEY,
-    uuid text UNIQUE NOT NULL,
-  owner integer REFERENCES users (id) ON DELETE RESTRICT,
-    -- The OpenVAS name for this credential.
-  name text NOT NULL,
-    -- The name of the account.
-  login text,
-    -- We have 3 uses for the password field:
-    -- 1. If private_key is NULL, this is a simple password.
-    -- 2. If private key is not NULL and its values is not ";;encrypted;;"
-    --  this is the passphrase to protect the private key.
-    -- 3. If private_key has the value ";;encrypted;;" this
-    --  is an encrypted container for a password or a private key.
-    --  The format of this container is a list of name-value pairs.
-    --  For example:
-    --   <len(8)> "password" <len> <value>
-    --   <len(10)> "private_key" <len> <value>
-    --  The len fields are encoded as 32 bit big endian unsigned
-    --  integer values.  A value of 0 for the value length is is
-    --  allowed and indicates and empty string.  A missing name
-    --  (e.g. "password", indicates a NULL value for that name.
-    --  This format closely resembles an unencrypted
-    --  lsc_credential.  However, recursive encryption is not
-    --  supported.  The container itself is OpenPGP encrypted
-    --  with the binary OpenPGP format wrapped into a standard
-    --  base64 encoding without linefeeds or checksums (i.e. it
-    --  is not the armor format).
-  password text,
-    -- A comment describing this credential.
-  comment text,
-    -- Private key, commonly used with ssh.
-    -- A flag value of ";;encrypted;;" in the private_key field is used to
-    -- indicate an encrypted credential.  Note: We can't use the password
-    -- field for the flag value because the password field is allowed to
-    -- contain arbitrary data; the private_key field however is a structured
-    -- value (for example PEM or plain base64).
-  private_key text,
-  creation_time date,
-  modification_time date);
-
---
--- Trashcan for lsc_credentials.
---
-CREATE TABLE lsc_credentials_trash
- (id SERIAL PRIMARY KEY,
-  uuid text UNIQUE NOT NULL,
-  owner integer REFERENCES users (id) ON DELETE RESTRICT,
-  name text NOT NULL,
-  login text,
-  password text,
-  comment text,
-  private_key text,
-  creation_time date,
-  modification_time date);
-
 CREATE TABLE notes
  (id SERIAL PRIMARY KEY,
   uuid text UNIQUE NOT NULL,
@@ -764,32 +851,6 @@ CREATE TABLE settings
   comment text,
   value text);
 
-CREATE TABLE slaves
- (id SERIAL PRIMARY KEY,
-  uuid text UNIQUE NOT NULL,
-  owner integer REFERENCES users (id) ON DELETE RESTRICT,
-  name text NOT NULL,
-  comment text,
-  host text,
-  port text,
-  login text,
-  password text,
-  creation_time date,
-  modification_time date);
-
-CREATE TABLE slaves_trash
- (id SERIAL PRIMARY KEY,
-  uuid text UNIQUE NOT NULL,
-  owner integer REFERENCES users (id) ON DELETE RESTRICT,
-  name text NOT NULL,
-  comment text,
-  host text,
-  port text,
-  login text,
-  password text,
-  creation_time date,
-  modification_time date);
-
 CREATE TABLE tags
  (id SERIAL PRIMARY KEY,
   uuid text UNIQUE NOT NULL,
