@@ -37,10 +37,6 @@
 
 #include <openvas/base/openvas_file.h>
 
-#ifdef S_SPLINT_S
-#include "splint.h"
-#endif
-
 #undef G_LOG_DOMAIN
 /**
  * @brief GLib log domain.
@@ -94,7 +90,7 @@ check_is_file (const char *name)
  *
  * @param[in]  comment     Comment to use.
  * @param[in]  passphrase  Passphrase for key, must be longer than 4 characters.
- * @param[in]  filepath    Path to file of public key (a .pub will be stripped).
+ * @param[in]  privpath    Filename of the key file.
  *
  * @return 0 if successful, -1 otherwise.
  */
@@ -427,7 +423,10 @@ lsc_user_rpm_recreate (const gchar *name, const char *public_key,
   int ret = -1;
 
   if (alien_found () == FALSE)
-    return -1;
+    {
+      g_warning ("%s: Need \"alien\" to make RPMs\n", __FUNCTION__);
+      return -1;
+    }
 
   /* Make a directory for the key. */
 
@@ -610,7 +609,10 @@ lsc_user_deb_recreate (const gchar *name, const char *rpm, gsize rpm_size,
   int ret = -1;
 
   if (alien_found () == FALSE)
-    return -1;
+    {
+      g_warning ("%s: Need \"alien\" to make DEBs\n", __FUNCTION__);
+      return -1;
+    }
 
   /* Make a directory for the RPM. */
 
@@ -851,12 +853,14 @@ lsc_user_exe_create (const gchar *user_name, const gchar *password,
 
   if (create_nsis_script (nsis_script, to_filename, user_name, password))
     {
+      g_warning ("%s: Failed to create NSIS script\n", __FUNCTION__);
       g_free (nsis_script);
       return -1;
     }
 
   if (execute_makensis (nsis_script))
     {
+      g_warning ("%s: Failed to execute makensis\n", __FUNCTION__);
       g_free (nsis_script);
       return -1;
     }
@@ -885,7 +889,10 @@ lsc_user_exe_recreate (const gchar *name, const gchar *password,
   int ret = -1;
 
   if (alien_found () == FALSE)
-    return -1;
+    {
+      g_warning ("%s: Need \"alien\" to make EXEs\n", __FUNCTION__);
+      return -1;
+    }
 
   /* Create NSIS package. */
 
