@@ -39,6 +39,12 @@
 #include <stdlib.h>
 #include <string.h>
 
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md manage"
+
 /**
  * @brief Test whether a user may perform an operation.
  *
@@ -47,12 +53,11 @@
  * @return 1 if user has permission, else 0.
  */
 int
-user_may (const char *operation)
+acl_user_may (const char *operation)
 {
   int ret;
   gchar *quoted_operation;
 
-  assert (current_credentials.uuid);
   assert (operation);
 
   if (strlen (current_credentials.uuid) == 0)
@@ -65,7 +70,7 @@ user_may (const char *operation)
 
   quoted_operation = sql_quote (operation);
 
-  ret = sql_int (USER_MAY ("0"),
+  ret = sql_int (ACL_USER_MAY ("0"),
                  current_credentials.uuid,
                  current_credentials.uuid,
                  current_credentials.uuid,
@@ -82,12 +87,12 @@ user_may (const char *operation)
 /**
  * @brief Check whether a role has Super Admin capability.
  *
- * @param[in]  uuid  Uuid of user.
+ * @param[in]  role_id  ID of role.
  *
  * @return 1 if role can Super Admin, else 0.
  */
 int
-role_can_super_everyone (const char *role_id)
+acl_role_can_super_everyone (const char *role_id)
 {
   gchar *quoted_role_id;
   quoted_role_id = sql_quote (role_id);
@@ -119,7 +124,7 @@ role_can_super_everyone (const char *role_id)
  * @return 1 if user is a Super Admin, else 0.
  */
 int
-user_can_super_everyone (const char *uuid)
+acl_user_can_super_everyone (const char *uuid)
 {
   gchar *quoted_uuid;
 
@@ -171,7 +176,7 @@ user_can_super_everyone (const char *uuid)
  * @return 1 if user has permission, else 0.
  */
 int
-user_can_everything (const char *user_id)
+acl_user_can_everything (const char *user_id)
 {
   gchar *quoted_user_id;
   int ret;
@@ -218,7 +223,7 @@ user_can_everything (const char *user_id)
  * @return 1 if user has permission, else 0.
  */
 int
-user_has_super (const char *super_user_id, user_t other_user)
+acl_user_has_super (const char *super_user_id, user_t other_user)
 {
   gchar *quoted_super_user_id;
 
@@ -288,7 +293,7 @@ user_has_super (const char *super_user_id, user_t other_user)
  * @return 1 if user is an Admin, else 0.
  */
 int
-user_is_admin (const char *uuid)
+acl_user_is_admin (const char *uuid)
 {
   int ret;
   gchar *quoted_uuid;
@@ -311,7 +316,7 @@ user_is_admin (const char *uuid)
  * @return 1 if user is an Observer, else 0.
  */
 int
-user_is_observer (const char *uuid)
+acl_user_is_observer (const char *uuid)
 {
   int ret;
   gchar *quoted_uuid;
@@ -357,7 +362,7 @@ user_is_super_admin (const char *uuid)
  * @return 1 if user has the User role, else 0.
  */
 int
-user_is_user (const char *uuid)
+acl_user_is_user (const char *uuid)
 {
   int ret;
   gchar *quoted_uuid;
@@ -377,7 +382,7 @@ user_is_user (const char *uuid)
  *
  * @param[in]  format  Value format specifier.
  */
-#define SUPER_CLAUSE(format)                                              \
+#define ACL_SUPER_CLAUSE(format)                                          \
   "                name = 'Super'"                                        \
   /*                    Super on everyone. */                             \
   "                AND ((resource = 0)"                                   \
@@ -438,7 +443,7 @@ user_is_user (const char *uuid)
  * @param[in]  user_id  UUID of user.
  * @param[in]  trash    Whether to search trash.
  */
-#define SUPER_CLAUSE_ARGS(type, field, value, user_id, trash) \
+#define ACL_SUPER_CLAUSE_ARGS(type, field, value, user_id, trash) \
   type,                                                       \
   trash ? (strcasecmp (type, "task") ? "_trash" : "") : "",   \
   type,                                                       \
@@ -472,15 +477,15 @@ user_is_user (const char *uuid)
  * @return 1 if user has Super, else 0.
  */
 int
-user_has_super_on (const char *type, const char *field, const char *value,
-                   int trash)
+acl_user_has_super_on (const char *type, const char *field, const char *value,
+                       int trash)
 {
   gchar *quoted_value;
   quoted_value = sql_quote (value);
   if (sql_int ("SELECT EXISTS (SELECT * FROM permissions"
-               "               WHERE " SUPER_CLAUSE ("'%s'") ");",
-               SUPER_CLAUSE_ARGS (type, field, quoted_value,
-                                  current_credentials.uuid, trash)))
+               "               WHERE " ACL_SUPER_CLAUSE ("'%s'") ");",
+               ACL_SUPER_CLAUSE_ARGS (type, field, quoted_value,
+                                      current_credentials.uuid, trash)))
     {
       g_free (quoted_value);
       return 1;
@@ -500,13 +505,13 @@ user_has_super_on (const char *type, const char *field, const char *value,
  * @return 1 if user has Super, else 0.
  */
 int
-user_has_super_on_resource (const char *type, const char *field,
-                            resource_t resource, int trash)
+acl_user_has_super_on_resource (const char *type, const char *field,
+                                resource_t resource, int trash)
 {
   if (sql_int ("SELECT EXISTS (SELECT * FROM permissions"
-               "               WHERE " SUPER_CLAUSE ("%llu") ");",
-               SUPER_CLAUSE_ARGS (type, field, resource,
-                                  current_credentials.uuid, trash)))
+               "               WHERE " ACL_SUPER_CLAUSE ("%llu") ");",
+               ACL_SUPER_CLAUSE_ARGS (type, field, resource,
+                                      current_credentials.uuid, trash)))
     return 1;
   return 0;
 }
@@ -517,13 +522,13 @@ user_has_super_on_resource (const char *type, const char *field,
  * A Super permissions can give a user effective ownership of another
  * user's resource.
  *
- * @param[in]  type  Type of resource, for example "report_format".
- * @param[in]  name  Name of resource.
+ * @param[in]  type   Type of resource, for example "report_format".
+ * @param[in]  value  Name of resource.
  *
  * @return 1 if user owns resource, else 0.
  */
 int
-user_owns_name (const char *type, const char *value)
+acl_user_owns_name (const char *type, const char *value)
 {
   gchar *quoted_value;
   int ret;
@@ -539,14 +544,15 @@ user_owns_name (const char *type, const char *value)
       || (strcmp (type, "dfn_cert_adv") == 0))
     return 1;
 
-  if (user_has_super_on (type, "name", value, 0))
+  if (acl_user_has_super_on (type, "name", value, 0))
     return 1;
 
   quoted_value = sql_quote (value);
   ret = sql_int ("SELECT count(*) FROM %ss"
                  " WHERE name = '%s'"
-                 " AND ((owner IS NULL) OR (owner ="
-                 " (SELECT users.id FROM users WHERE users.uuid = '%s')));",
+                 " AND ((owner IS NULL)"
+                 "      OR (owner = (SELECT users.id FROM users"
+                 "                   WHERE users.uuid = '%s')));",
                  type,
                  quoted_value,
                  current_credentials.uuid);
@@ -564,7 +570,7 @@ user_owns_name (const char *type, const char *value)
  * @return 1 if user actually owns resource, else 0.
  */
 int
-user_is_owner (const char *type, const char *uuid)
+acl_user_is_owner (const char *type, const char *uuid)
 {
   int ret;
   gchar *quoted_uuid;
@@ -597,7 +603,7 @@ user_is_owner (const char *type, const char *uuid)
  * @return 1 if user owns resource, else 0.
  */
 int
-user_owns_uuid (const char *type, const char *uuid, int trash)
+acl_user_owns_uuid (const char *type, const char *uuid, int trash)
 {
   int ret;
   gchar *quoted_uuid;
@@ -612,7 +618,7 @@ user_owns_uuid (const char *type, const char *uuid, int trash)
       || (strcmp (type, "dfn_cert_adv") == 0))
     return 1;
 
-  if (user_has_super_on (type, "uuid", uuid, 0))
+  if (acl_user_has_super_on (type, "uuid", uuid, 0))
     return 1;
 
   quoted_uuid = sql_quote (uuid);
@@ -620,16 +626,18 @@ user_owns_uuid (const char *type, const char *uuid, int trash)
     ret = sql_int ("SELECT count(*) FROM results, reports"
                    " WHERE results.uuid = '%s'"
                    " AND results.report = reports.id"
-                   " AND ((reports.owner IS NULL) OR (reports.owner ="
-                   " (SELECT users.id FROM users WHERE users.uuid = '%s')));",
+                   " AND ((reports.owner IS NULL)"
+                   "      OR (reports.owner = (SELECT users.id FROM users"
+                   "                           WHERE users.uuid = '%s')));",
                    quoted_uuid,
                    current_credentials.uuid);
   else
     ret = sql_int ("SELECT count(*) FROM %ss%s"
                    " WHERE uuid = '%s'"
                    "%s"
-                   " AND ((owner IS NULL) OR (owner ="
-                   " (SELECT users.id FROM users WHERE users.uuid = '%s')));",
+                   " AND ((owner IS NULL)"
+                   "      OR (owner = (SELECT users.id FROM users"
+                   "                   WHERE users.uuid = '%s')));",
                    type,
                    (strcmp (type, "task") && trash) ? "_trash" : "",
                    quoted_uuid,
@@ -655,7 +663,7 @@ user_owns_uuid (const char *type, const char *uuid, int trash)
  * @return 1 if user owns resource, else 0.
  */
 int
-user_owns (const char *type, resource_t resource, int trash)
+acl_user_owns (const char *type, resource_t resource, int trash)
 {
   int ret;
 
@@ -669,23 +677,25 @@ user_owns (const char *type, resource_t resource, int trash)
       || (strcmp (type, "dfn_cert_adv") == 0))
     return 1;
 
-  if (user_has_super_on_resource (type, "id", resource, trash))
+  if (acl_user_has_super_on_resource (type, "id", resource, trash))
     return 1;
 
   if (strcmp (type, "result") == 0)
     ret = sql_int ("SELECT count(*) FROM results, reports"
                    " WHERE results.id = %llu"
                    " AND results.report = reports.id"
-                   " AND ((reports.owner IS NULL) OR (reports.owner ="
-                   " (SELECT users.id FROM users WHERE users.uuid = '%s')));",
+                   " AND ((reports.owner IS NULL)"
+                   "      OR (reports.owner = (SELECT users.id FROM users"
+                   "                           WHERE users.uuid = '%s')));",
                    resource,
                    current_credentials.uuid);
   else
     ret = sql_int ("SELECT count(*) FROM %ss%s"
                    " WHERE id = %llu"
                    "%s"
-                   " AND ((owner IS NULL) OR (owner ="
-                   " (SELECT users.id FROM users WHERE users.uuid = '%s')));",
+                   " AND ((owner IS NULL)"
+                   "      OR (owner = (SELECT users.id FROM users"
+                   "                   WHERE users.uuid = '%s')));",
                    type,
                    (strcmp (type, "task") && trash) ? "_trash" : "",
                    resource,
@@ -709,7 +719,7 @@ user_owns (const char *type, resource_t resource, int trash)
  * @return 1 if user owns resource, else 0.
  */
 int
-user_owns_trash_uuid (const char *type, const char *uuid)
+acl_user_owns_trash_uuid (const char *type, const char *uuid)
 {
   int ret;
   gchar *quoted_uuid;
@@ -717,14 +727,15 @@ user_owns_trash_uuid (const char *type, const char *uuid)
   assert (current_credentials.uuid);
   assert (type && strcmp (type, "task"));
 
-  if (user_has_super_on (type, "uuid", uuid, 1))
+  if (acl_user_has_super_on (type, "uuid", uuid, 1))
     return 1;
 
   quoted_uuid = sql_quote (uuid);
   ret = sql_int ("SELECT count(*) FROM %ss_trash"
                  " WHERE uuid = '%s'"
-                 " AND ((owner IS NULL) OR (owner ="
-                 " (SELECT users.id FROM users WHERE users.uuid = '%s')));",
+                 " AND ((owner IS NULL)"
+                 "      OR (owner = (SELECT users.id FROM users"
+                 "                   WHERE users.uuid = '%s')));",
                  type,
                  quoted_uuid,
                  current_credentials.uuid);
@@ -744,8 +755,8 @@ user_owns_trash_uuid (const char *type, const char *uuid)
  * @return 1 if user may access resource, else 0.
  */
 int
-user_has_access_uuid (const char *type, const char *uuid,
-                      const char *permission, int trash)
+acl_user_has_access_uuid (const char *type, const char *uuid,
+                          const char *permission, int trash)
 {
   int ret, get;
   char *uuid_task;
@@ -760,7 +771,7 @@ user_has_access_uuid (const char *type, const char *uuid,
     return 1;
 
   /* The Super case is checked here. */
-  ret = user_owns_uuid (type, uuid, trash);
+  ret = acl_user_owns_uuid (type, uuid, trash);
   if (ret)
     return ret;
 
@@ -934,9 +945,10 @@ user_has_access_uuid (const char *type, const char *uuid,
  * @return Newly allocated owned clause.
  */
 gchar *
-where_owned_user (const char *user_id, const char *user_sql, const char *type,
-                  const get_data_t *get, int owned, const gchar *owner_filter,
-                  resource_t resource, array_t *permissions)
+acl_where_owned_user (const char *user_id, const char *user_sql,
+                      const char *type, const get_data_t *get, int owned,
+                      const gchar *owner_filter, resource_t resource,
+                      array_t *permissions)
 {
   gchar *owned_clause;
 
@@ -944,7 +956,8 @@ where_owned_user (const char *user_id, const char *user_sql, const char *type,
     {
       gchar *permission_clause, *filter_owned_clause;
       GString *permission_or;
-      int index, table_trash, permissions_include_get;
+      int table_trash, permissions_include_get;
+      guint index;
 
       permissions_include_get = 0;
       permission_or = g_string_new ("");
@@ -984,6 +997,7 @@ where_owned_user (const char *user_id, const char *user_sql, const char *type,
 
       /* Check on index is because default is owner and global, for backward
        * compatibility. */
+      permission_clause = NULL;
       if (user_id && index)
         {
           gchar *clause;
@@ -1089,8 +1103,6 @@ where_owned_user (const char *user_id, const char *user_sql, const char *type,
           else
             permission_clause = clause;
         }
-      else
-        permission_clause = NULL;
 
       g_string_free (permission_or, TRUE);
 
@@ -1102,7 +1114,7 @@ where_owned_user (const char *user_id, const char *user_sql, const char *type,
         {
           int admin;
           assert (strcmp (user_id, ""));
-          admin = user_can_everything (user_id);
+          admin = acl_user_can_everything (user_id);
           /* A user sees permissions that involve the user.  Admin users also
            * see all higher level permissions. */
           owned_clause
@@ -1207,7 +1219,7 @@ where_owned_user (const char *user_id, const char *user_sql, const char *type,
         }
       else
         owned_clause
-         = g_strdup_printf (/* Either a global resource (like target Localhost).
+         = g_strdup_printf (/* Either a global resource (like Full and Fast).
                              *
                              * The globals are only required when GET permission
                              * is requested, because it is only possible to read
@@ -1337,9 +1349,9 @@ where_owned_user (const char *user_id, const char *user_sql, const char *type,
  * @return Newly allocated owned clause.
  */
 gchar *
-where_owned (const char *type, const get_data_t *get, int owned,
-             const gchar *owner_filter, resource_t resource,
-             array_t *permissions)
+acl_where_owned (const char *type, const get_data_t *get, int owned,
+                 const gchar *owner_filter, resource_t resource,
+                 array_t *permissions)
 {
   gchar *ret, *user_sql;
   if (current_credentials.uuid)
@@ -1347,8 +1359,8 @@ where_owned (const char *type, const get_data_t *get, int owned,
                                 current_credentials.uuid);
   else
     user_sql = NULL;
-  ret = where_owned_user (current_credentials.uuid, user_sql, type, get, owned,
-                          owner_filter, resource, permissions);
+  ret = acl_where_owned_user (current_credentials.uuid, user_sql, type, get,
+                              owned, owner_filter, resource, permissions);
   g_free (user_sql);
   return ret;
 }
@@ -1363,7 +1375,7 @@ where_owned (const char *type, const get_data_t *get, int owned,
  * @return Newly allocated owned clause.
  */
 gchar *
-where_owned_for_get (const char *type, const char *user_sql)
+acl_where_owned_for_get (const char *type, const char *user_sql)
 {
   gchar *owned_clause;
   get_data_t get;
@@ -1381,17 +1393,17 @@ where_owned_for_get (const char *type, const char *user_sql)
   get.trash = 0;
   permissions = make_array ();
   array_add (permissions, g_strdup_printf ("get_%ss", type));
-  owned_clause = where_owned_user (current_credentials.uuid
-                                    ? current_credentials.uuid
-                                    /* Use user_sql_new. */
-                                    : "",
-                                   user_sql_new,
-                                   type,
-                                   &get,
-                                   1,              /* Do owner checks. */
-                                   "any",
-                                   0,              /* Resource. */
-                                   permissions);
+  owned_clause = acl_where_owned_user (current_credentials.uuid
+                                        ? current_credentials.uuid
+                                        /* Use user_sql_new. */
+                                        : "",
+                                       user_sql_new,
+                                       type,
+                                       &get,
+                                       1,              /* Do owner checks. */
+                                       "any",
+                                       0,              /* Resource. */
+                                       permissions);
   array_free (permissions);
   g_free (user_sql_new);
 
