@@ -26,6 +26,12 @@
 #include "manage.h"
 #include "sql.h"
 
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md   main"
+
 /**
  * @brief Insert a nvt selector.
  */
@@ -41,25 +47,11 @@
  *
  * Caller must lock the db.
  *
- * @param[in]  uuid           UUID for new scan config.
  * @param[in]  selector_name  Name of NVT selector to use.
  */
 void
-make_config_discovery (char *const uuid, char *const selector_name)
+make_config_discovery_service_detection (char *const selector_name)
 {
-  sql_begin_exclusive ();
-
-  /* First, create the Discovery config. */
-  sql ("INSERT into configs (uuid, name, owner, nvt_selector, comment,"
-       " family_count, nvt_count, nvts_growing, families_growing,"
-       " type, creation_time, modification_time)"
-       " VALUES ('%s', 'Discovery', NULL,"
-       "         '%s', 'Network Discovery scan configuration.',"
-       "         0, 0, 0, 0, 0, m_now (), m_now ());",
-       uuid,
-       selector_name);
-
-  /* Setup the appropriate NVTs for the config. */
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.11929", "Service detection");
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.900534", "Service detection");
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.902019", "Service detection");
@@ -576,6 +568,31 @@ make_config_discovery (char *const uuid, char *const selector_name)
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.900371", "Service detection");
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.900374", "Service detection");
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.900376", "Service detection");
+}
+
+/**
+ * @brief Make Discovery Scan Config.
+ *
+ * Caller must lock the db.
+ *
+ * @param[in]  uuid           UUID for new scan config.
+ * @param[in]  selector_name  Name of NVT selector to use.
+ */
+void
+make_config_discovery (char *const uuid, char *const selector_name)
+{
+  /* First, create the Discovery config. */
+  sql ("INSERT into configs (uuid, name, owner, nvt_selector, comment,"
+       " family_count, nvt_count, nvts_growing, families_growing,"
+       " type, creation_time, modification_time)"
+       " VALUES ('%s', 'Discovery', NULL,"
+       "         '%s', 'Network Discovery scan configuration.',"
+       "         0, 0, 0, 0, 0, m_now (), m_now ());",
+       uuid,
+       selector_name);
+
+  /* Setup the appropriate NVTs for the config. */
+  make_config_discovery_service_detection (selector_name);
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.17586", "Brute force attacks");
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.17584", "Brute force attacks");
   NVT_SELECTOR (selector_name, "1.3.6.1.4.1.25623.1.0.17368", "Brute force attacks");
@@ -931,7 +948,7 @@ make_config_discovery (char *const uuid, char *const selector_name)
        " VALUES ((SELECT id FROM configs WHERE uuid = '%s'),"
        "         'PLUGINS_PREFS',"
        "         'Ping Host[checkbox]:Report about unrechable Hosts',"
-       "         'yes');",
+       "         'no');",
        uuid);
 
   /* Add preferences for "Services" nvt in Discovery Scan Config. */
@@ -941,7 +958,26 @@ make_config_discovery (char *const uuid, char *const selector_name)
        "         'Services[radio]:Test SSL based services',"
        "         'All;Known SSL ports;None');",
        uuid);
+}
 
+/**
+ * @brief Ensure the Discovery config is up to date.
+ *
+ * @param[in]  uuid  UUID of config.
+ *
+ * @return 0 success, -1 error.
+ */
+int
+check_config_discovery (const char *uuid)
+{
+  /* Check new preference. */
 
-  sql ("COMMIT;");
+  sql ("UPDATE config_preferences SET value = 'no'"
+       " WHERE config = (SELECT id FROM configs WHERE uuid = '%s')"
+       " AND type = 'PLUGINS_PREFS'"
+       " AND name = 'Ping Host[checkbox]:Report about unrechable Hosts'"
+       " AND value = 'yes';",
+       uuid);
+
+  return 0;
 }
