@@ -23,22 +23,19 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "manage.h"
-#include "sql.h"
+#include "manage_sql.h"
+
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md   main"
 
 /**
  * @brief Insert a port range.
  */
 #define RANGE(type, start, end)                                      \
-  sql ("INSERT INTO port_ranges"                                     \
-       " (uuid, port_list, type, start, \"end\", comment, exclude)"  \
-       " VALUES"                                                     \
-       " (make_uuid (), %llu, %i,"                                   \
-       "  '" G_STRINGIFY (start) "',"                                \
-       "  '" G_STRINGIFY (end) "',"                                  \
-       "  '', 0)",                                                   \
-       list,                                                         \
-       type)
+  insert_port_range (list, type, start, end)
 
 /**
  * @brief Make port ranges for IANA TCP 2012.
@@ -50,8 +47,6 @@
 void
 make_port_ranges_all_tcp_nmap_5_51_top_1000 (port_list_t list)
 {
-  sql_begin_immediate ();
-
   RANGE (PORT_PROTOCOL_TCP, 1, 65535);
 
   RANGE (PORT_PROTOCOL_UDP, 2, 3);
@@ -838,6 +833,4 @@ make_port_ranges_all_tcp_nmap_5_51_top_1000 (port_list_t list)
   RANGE (PORT_PROTOCOL_UDP, 64513, 64513);
   RANGE (PORT_PROTOCOL_UDP, 64590, 64590);
   RANGE (PORT_PROTOCOL_UDP, 64727, 64727);
-
-  sql ("END TRANSACTION");
 }
