@@ -23,22 +23,19 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "manage.h"
-#include "sql.h"
+#include "manage_sql.h"
+
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md   main"
 
 /**
  * @brief Insert a port range.
  */
 #define RANGE(type, start, end)                                      \
-  sql ("INSERT INTO port_ranges"                                     \
-       " (uuid, port_list, type, start, \"end\", comment, exclude)"  \
-       " VALUES"                                                     \
-       " (make_uuid (), %llu, %i,"                                   \
-       "  '" G_STRINGIFY (start) "',"                                \
-       "  '" G_STRINGIFY (end) "',"                                  \
-       "  '', 0)",                                                   \
-       list,                                                         \
-       type)
+  insert_port_range (list, type, start, end)
 
 /**
  * @brief Make port ranges for IANA TCP 2012.
@@ -50,8 +47,6 @@
 void
 make_port_ranges_iana_tcp_2012 (port_list_t list)
 {
-  sql_begin_immediate ();
-
   RANGE (PORT_PROTOCOL_TCP, 1, 80);
   RANGE (PORT_PROTOCOL_TCP, 82, 113);
   RANGE (PORT_PROTOCOL_TCP, 115, 224);
@@ -731,6 +726,4 @@ make_port_ranges_iana_tcp_2012 (port_list_t list)
   RANGE (PORT_PROTOCOL_TCP, 48128, 48129);
   RANGE (PORT_PROTOCOL_TCP, 48556, 48556);
   RANGE (PORT_PROTOCOL_TCP, 48619, 48619);
-
-  sql ("END TRANSACTION");
 }
