@@ -23,22 +23,19 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#include "manage.h"
-#include "sql.h"
+#include "manage_sql.h"
+
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md   main"
 
 /**
  * @brief Insert a port range.
  */
 #define RANGE(type, start, end)                                      \
-  sql ("INSERT INTO port_ranges"                                     \
-       " (uuid, port_list, type, start, \"end\", comment, exclude)"  \
-       " VALUES"                                                     \
-       " (make_uuid (), %llu, %i,"                                   \
-       "  '" G_STRINGIFY (start) "',"                                \
-       "  '" G_STRINGIFY (end) "',"                                  \
-       "  '', 0)",                                                   \
-       list,                                                         \
-       type)
+  insert_port_range (list, type, start, end)
 
 /**
  * @brief Make port ranges for Nmap top 2000 top 100.
@@ -50,8 +47,6 @@
 void
 make_port_ranges_nmap_5_51_top_2000_top_100 (port_list_t list)
 {
-  sql_begin_immediate ();
-
   RANGE (PORT_PROTOCOL_TCP, 1, 1);
   RANGE (PORT_PROTOCOL_TCP, 3, 4);
   RANGE (PORT_PROTOCOL_TCP, 6, 7);
@@ -1263,6 +1258,4 @@ make_port_ranges_nmap_5_51_top_2000_top_100 (port_list_t list)
   RANGE (PORT_PROTOCOL_UDP, 49188, 49188);
   RANGE (PORT_PROTOCOL_UDP, 49190, 49194);
   RANGE (PORT_PROTOCOL_UDP, 49200, 49201);
-
-  sql ("END TRANSACTION");
 }
