@@ -44,7 +44,6 @@
 /** @todo For scanner_init_state. */
 #include "otp.h"
 #include "ovas-mngr-comm.h"
-#include "tracef.h"
 
 #include <assert.h>
 #include <dirent.h>
@@ -59,18 +58,15 @@
 
 #include <openvas/misc/openvas_server.h>
 
-#ifdef S_SPLINT_S
-/** @todo Weird that these are missing. */
-/*@-exportheader@*/
-int socket(int domain, int type, int protocol);
-/*@=exportheader@*/
-#endif
-
-#ifndef S_SPLINT_S
 #if FROM_BUFFER_SIZE > SSIZE_MAX
 #error FROM_BUFFER_SIZE too big for "read"
 #endif
-#endif
+
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md   main"
 
 /**
  * @brief Buffer of input from the client.
@@ -107,8 +103,10 @@ static int ompd_nvt_cache_mode = 0;
  * @param[in]  max_ips_per_target  Max number of IPs per target.
  * @param[in]  max_email_attachment_size  Max size of email attachments.
  * @param[in]  max_email_include_size     Max size of email inclusions.
- * @param[in]  progress        Function to update progress, or NULL.
- * @param[in]  skip_db_check   Skip DB check.
+ * @param[in]  progress         Function to update progress, or NULL.
+ * @param[in]  fork_connection  Function to fork a connection to the OMP
+ *                              daemon layer, or NULL.
+ * @param[in]  skip_db_check    Skip DB check.
  *
  * @return 0 success, -1 error, -2 database is wrong version, -3 database
  *         needs to be initialized from server, -4 max_ips_per_target out of
@@ -117,11 +115,13 @@ static int ompd_nvt_cache_mode = 0;
 int
 init_ompd (GSList *log_config, int nvt_cache_mode, const gchar *database,
            int max_ips_per_target, int max_email_attachment_size,
-           int max_email_include_size, void (*progress) (), int skip_db_check)
+           int max_email_include_size, void (*progress) (),
+           int (*fork_connection) (openvas_connection_t *, gchar*),
+           int skip_db_check)
 {
   return init_omp (log_config, nvt_cache_mode, database, max_ips_per_target,
                    max_email_attachment_size, max_email_include_size,
-                   progress, skip_db_check);
+                   progress, fork_connection, skip_db_check);
 }
 
 /**
@@ -133,21 +133,61 @@ init_ompd (GSList *log_config, int nvt_cache_mode, const gchar *database,
 void
 init_ompd_process (const gchar *database, gchar **disable)
 {
+  openvas_scanner_fork ();
+  from_client_start = 0;
+  from_client_end = 0;
   init_omp_process (0, database, NULL, NULL, disable);
 }
 
+/**
+ * @brief Read as much from the client as the \ref from_client buffer will hold.
+ *
+ * @param[in]  client_socket  The socket.
+ *
+ * @return 0 on reading everything available, -1 on error, -2 if
+ *         from_client buffer is full or -3 on reaching end of file.
+ */
+static int
+read_from_client_unix (int client_socket)
+{
+  while (from_client_end < from_buffer_size)
+    {
+      int count;
+      count = read (client_socket,
+                    from_client + from_client_end,
+                    from_buffer_size - from_client_end);
+      if (count < 0)
+        {
+          if (errno == EAGAIN)
+            /* Got everything available, return to `select'. */
+            return 0;
+          if (errno == EINTR)
+            /* Interrupted, try read again. */
+            continue;
+          g_warning ("%s: failed to read from client: %s\n",
+                     __FUNCTION__, strerror (errno));
+          return -1;
+        }
+      if (count == 0)
+        /* End of file. */
+        return -3;
+      from_client_end += count;
+    }
+
+  /* Buffer full. */
+  return -2;
+}
+
 /**
  * @brief Read as much from the client as the \ref from_client buffer will hold.
  *
  * @param[in]  client_session  The TLS session with the client.
- * @param[in]  client_socket   The socket connected to the client.
  *
  * @return 0 on reading everything available, -1 on error, -2 if
  * from_client buffer is full or -3 on reaching end of file.
  */
 static int
-read_from_client (gnutls_session_t* client_session,
-                  /*@unused@*/ int client_socket)
+read_from_client_tls (gnutls_session_t* client_session)
 {
   while (from_client_end < from_buffer_size)
     {
@@ -166,7 +206,7 @@ read_from_client (gnutls_session_t* client_session,
           if (count == GNUTLS_E_REHANDSHAKE)
             {
               /** @todo Rehandshake. */
-              tracef ("   should rehandshake\n");
+              g_debug ("   should rehandshake\n");
               continue;
             }
           if (gnutls_error_is_fatal ((int) count) == 0
@@ -174,7 +214,6 @@ read_from_client (gnutls_session_t* client_session,
                   || count == GNUTLS_E_FATAL_ALERT_RECEIVED))
             {
               int alert = gnutls_alert_get (*client_session);
-              /*@dependent@*/
               const char* alert_name = gnutls_alert_get_name (alert);
               g_warning ("%s: TLS Alert %d: %s\n",
                          __FUNCTION__, alert, alert_name);
@@ -193,6 +232,22 @@ read_from_client (gnutls_session_t* client_session,
   return -2;
 }
 
+/**
+ * @brief Read as much from the client as the \ref from_client buffer will hold.
+ *
+ * @param[in]  client_connection  The connection with the client.
+ *
+ * @return 0 on reading everything available, -1 on error, -2 if
+ * from_client buffer is full or -3 on reaching end of file.
+ */
+static int
+read_from_client (openvas_connection_t *client_connection)
+{
+  if (client_connection->tls)
+    return read_from_client_tls (&client_connection->session);
+  return read_from_client_unix (client_connection->socket);
+}
+
 /** @todo Move to openvas-libraries? */
 /**
  * @brief Write as much as possible from \ref to_client to the client.
@@ -202,7 +257,7 @@ read_from_client (gnutls_session_t* client_session,
  * @return 0 wrote everything, -1 error, -2 wrote as much as client accepted.
  */
 static int
-write_to_client (gnutls_session_t* client_session)
+write_to_client_tls (gnutls_session_t* client_session)
 {
   while (to_client_start < to_client_end)
     {
@@ -230,15 +285,72 @@ write_to_client (gnutls_session_t* client_session)
             to_client_end - to_client_start,
             to_client + to_client_start);
       to_client_start += count;
-      tracef ("=> client  %u bytes\n", (unsigned int) count);
+      g_debug ("=> client  %u bytes\n", (unsigned int) count);
     }
-  tracef ("=> client  done\n");
+  g_debug ("=> client  done\n");
   to_client_start = to_client_end = 0;
 
   /* Wrote everything. */
   return 0;
 }
 
+/**
+ * @brief Write as much as possible from \ref to_client to the client.
+ *
+ * @param[in]  client_socket  The client socket.
+ *
+ * @return 0 wrote everything, -1 error, -2 wrote as much as client accepted.
+ */
+static int
+write_to_client_unix (int client_socket)
+{
+  while (to_client_start < to_client_end)
+    {
+      ssize_t count;
+      count = write (client_socket,
+                     to_client + to_client_start,
+                     to_client_end - to_client_start);
+      if (count < 0)
+        {
+          if (errno == EAGAIN)
+            /* Wrote as much as client would accept. */
+            return -2;
+          if (errno == EINTR)
+            /* Interrupted, try write again. */
+            continue;
+          g_warning ("%s: failed to write to client: %s\n",
+                     __FUNCTION__,
+                     strerror (errno));
+          return -1;
+        }
+      logf ("=> client %.*s\n",
+            to_client_end - to_client_start,
+            to_client + to_client_start);
+      to_client_start += count;
+      g_debug ("=> client  %u bytes\n", (unsigned int) count);
+    }
+  g_debug ("=> client  done\n");
+  to_client_start = to_client_end = 0;
+
+  /* Wrote everything. */
+  return 0;
+}
+
+/**
+ * @brief Write as much as possible from \ref to_client to the client.
+ *
+ * @param[in]  client_connection  The client connection.
+ *
+ * @return 0 wrote everything, -1 error, -2 wrote as much as client accepted.
+ */
+static int
+write_to_client (openvas_connection_t *client_connection)
+{
+  if (client_connection->tls)
+    return write_to_client_tls (&client_connection->session);
+  return write_to_client_unix (client_connection->socket);
+}
+
 /**
  * @brief Send a response message to the client.
  *
@@ -267,7 +379,7 @@ ompd_send_to_client (const char* msg, void* write_to_client_data)
           case  0:      /* Wrote everything in to_client. */
             break;
           case -1:      /* Error. */
-            tracef ("   %s full (%i < %zu); client write failed\n",
+            g_debug ("   %s full (%i < %zu); client write failed\n",
                     __FUNCTION__,
                     ((buffer_size_t) TO_CLIENT_BUFFER_SIZE) - to_client_end,
                     strlen (msg));
@@ -284,7 +396,7 @@ ompd_send_to_client (const char* msg, void* write_to_client_data)
         break;
 
       memmove (to_client + to_client_end, msg, length);
-      tracef ("-> client: %.*s\n", (int) length, msg);
+      g_debug ("-> client: %.*s\n", (int) length, msg);
       to_client_end += length;
       msg += length;
     }
@@ -294,25 +406,30 @@ ompd_send_to_client (const char* msg, void* write_to_client_data)
       assert (strlen (msg)
               <= (((buffer_size_t) TO_CLIENT_BUFFER_SIZE) - to_client_end));
       memmove (to_client + to_client_end, msg, strlen (msg));
-      tracef ("-> client: %s\n", msg);
+      g_debug ("-> client: %s\n", msg);
       to_client_end += strlen (msg);
     }
 
   return FALSE;
 }
 
+/**
+ * @brief Clean session.
+ *
+ * @param[in]  client_connection  Connection.
+ */
 static void
-session_clean (gnutls_session_t *sess, gnutls_certificate_credentials_t *creds)
+session_clean (openvas_connection_t *client_connection)
 {
-  if (sess && *sess)
+  if (client_connection->session)
     {
-      gnutls_deinit (*sess);
-      *sess = NULL;
+      gnutls_deinit (client_connection->session);
+      client_connection->session = NULL;
     }
-  if (creds && *creds)
+  if (client_connection->credentials)
     {
-      gnutls_certificate_free_credentials (*creds);
-      *creds = NULL;
+      gnutls_certificate_free_credentials (client_connection->credentials);
+      client_connection->credentials = NULL;
     }
 }
 
@@ -340,20 +457,16 @@ session_clean (gnutls_session_t *sess, gnutls_certificate_credentials_t *creds)
  *
  * If client_socket is 0 or less, then update the NVT cache and exit.
  *
- * @param[in]  client_session       The TLS session with the client.
- * @param[in]  client_credentials   The TSL client credentials.
- * @param[in]  client_socket        The socket connected to the client, if any.
+ * @param[in]  client_connection    Connection.
  * @param[in]  database             Location of manage database.
  * @param[in]  disable              Commands to disable.
  * @param[in]  progress             Function to mark progress, or NULL.
  *
- * @return 0 on success, -1 on error, 1 scanner still loading.
+ * @return 0 success, 1 scanner still loading, -1 error, -2 scanner has no cert.
  */
 int
-serve_omp (gnutls_session_t* client_session,
-           gnutls_certificate_credentials_t* client_credentials,
-           int client_socket, const gchar* database, gchar **disable,
-           void (*progress) ())
+serve_omp (openvas_connection_t *client_connection, const gchar *database,
+           gchar **disable, void (*progress) ())
 {
   int nfds, scan_handler = 0, rc = 0;
   /* True if processing of the client input is waiting for space in the
@@ -361,21 +474,21 @@ serve_omp (gnutls_session_t* client_session,
   short client_input_stalled;
   /* Client status flag.  Set to 0 when the client closes the connection
    * while the scanner is active. */
-  short client_active = client_socket > 0;
+  short client_active = client_connection->socket > 0;
 
-  if (client_socket < 0)
-    ompd_nvt_cache_mode = client_socket;
+  if (client_connection->socket < 0)
+    ompd_nvt_cache_mode = client_connection->socket;
 
   if (ompd_nvt_cache_mode)
-    infof ("   Updating NVT cache.\n");
+    g_info ("   Updating NVT cache.\n");
   else
-    tracef ("   Serving OMP.\n");
+    g_debug ("   Serving OMP.\n");
 
   /* Initialise the XML parser and the manage library. */
   init_omp_process (ompd_nvt_cache_mode,
                     database,
                     (int (*) (const char*, void*)) ompd_send_to_client,
-                    (void*) client_session,
+                    (void*) client_connection,
                     disable);
 #if 0
   /** @todo Consider free_omp_data (); on return. */
@@ -388,11 +501,15 @@ serve_omp (gnutls_session_t* client_session,
   /* Setup the scanner address and try to connect. */
   if (ompd_nvt_cache_mode && !openvas_scanner_connected ())
     {
+      int ret;
+
       /* Is here because it queries the DB and needs it initialized.
        * XXX: Move outside serve_omp ().
        */
-      if (manage_scanner_set_default ())
-        return -1;
+
+      ret = manage_scanner_set_default ();
+      if (ret)
+        return ret;
       if (openvas_scanner_connect () || openvas_scanner_init (1))
         {
           openvas_scanner_close ();
@@ -434,7 +551,7 @@ serve_omp (gnutls_session_t* client_session,
    *     to_scanner buffer filling up (during initialisation).
    */
 
-  nfds = openvas_scanner_get_nfds (client_socket);
+  nfds = openvas_scanner_get_nfds (client_connection->socket);
   while (1)
     {
       int ret;
@@ -453,10 +570,10 @@ serve_omp (gnutls_session_t* client_session,
         {
           /* See whether to read from the client.  */
           if (from_client_end < from_buffer_size)
-            FD_SET (client_socket, &readfds);
+            FD_SET (client_connection->socket, &readfds);
           /* See whether to write to the client.  */
           if (to_client_start < to_client_end)
-            FD_SET (client_socket, &writefds);
+            FD_SET (client_connection->socket, &writefds);
         }
 
       /* See whether we need to read from the scannner.  */
@@ -489,13 +606,15 @@ serve_omp (gnutls_session_t* client_session,
        * exhibit a problem in OpenVAS due to a different buffering
        * strategy.  */
       ret = 0;
-      if (client_socket > 0 && FD_ISSET (client_socket, &readfds)
-          && gnutls_record_check_pending (*client_session))
+      if (client_connection->socket > 0
+          && client_connection->tls
+          && FD_ISSET (client_connection->socket, &readfds)
+          && gnutls_record_check_pending (client_connection->session))
         {
           FD_ZERO (&readfds);
           FD_ZERO (&writefds);
           ret++;
-          FD_SET (client_socket, &readfds);
+          FD_SET (client_connection->socket, &readfds);
         }
       if (openvas_scanner_fd_isset (&readfds))
         {
@@ -546,12 +665,12 @@ serve_omp (gnutls_session_t* client_session,
         }
 
       /* Read any data from the client. */
-      if (client_socket > 0 && FD_ISSET (client_socket, &readfds))
+      if (client_connection->socket > 0
+          && FD_ISSET (client_connection->socket, &readfds))
         {
-#if TRACE || LOG
           buffer_size_t initial_start = from_client_end;
-#endif
-          switch (read_from_client (client_session, client_socket))
+
+          switch (read_from_client (client_connection))
             {
               case  0:       /* Read everything. */
                 break;
@@ -562,14 +681,13 @@ serve_omp (gnutls_session_t* client_session,
                 /* There may be more to read. */
                 break;
               case -3:       /* End of file. */
-                tracef ("   EOF reading from client.\n");
+                g_debug ("   EOF reading from client.\n");
                 rc = 0;
                 goto client_free;
               default:       /* Programming error. */
                 assert (0);
             }
 
-#if TRACE || LOG
           /* This check prevents output in the "asynchronous network
            * error" case. */
           if (from_client_end > initial_start)
@@ -577,21 +695,15 @@ serve_omp (gnutls_session_t* client_session,
               logf ("<= client %.*s\n",
                     from_client_end - initial_start,
                     from_client + initial_start);
-#if TRACE_TEXT
               if (g_strstr_len (from_client + initial_start,
                                 from_client_end - initial_start,
                                 "<password>"))
-                tracef ("<= client  Input may contain password, suppressed.\n");
+                g_debug ("<= client  Input may contain password, suppressed.\n");
               else
-                tracef ("<= client  \"%.*s\"\n",
+                g_debug ("<= client  \"%.*s\"\n",
                         from_client_end - initial_start,
                         from_client + initial_start);
-#else
-              tracef ("<= client  %i bytes\n",
-                      from_client_end - initial_start);
-#endif
             }
-#endif /* TRACE || LOG */
 
           ret = process_omp_client_input ();
           if (ret == 0)
@@ -603,7 +715,7 @@ serve_omp (gnutls_session_t* client_session,
                * without closing it, for usage by the child process. */
               set_scanner_init_state (SCANNER_INIT_TOP);
               openvas_scanner_free ();
-              nfds = openvas_scanner_get_nfds (client_socket);
+              nfds = openvas_scanner_get_nfds (client_connection->socket);
               client_input_stalled = 0;
               /* Skip the rest of the loop because the scanner socket is
                * a new socket.  This is asking for select trouble, really. */
@@ -615,7 +727,7 @@ serve_omp (gnutls_session_t* client_session,
                * successfully started the task.  Close the client
                * connection, as the parent process has continued the
                * session with the client. */
-              session_clean (client_session, client_credentials);
+              session_clean (client_connection);
               client_active = 0;
               client_input_stalled = 0;
               scan_handler = 1;
@@ -626,14 +738,14 @@ serve_omp (gnutls_session_t* client_session,
                * successfully completed.  Close the client connection,
                * and exit, as the parent process has continued the
                * session with the client. */
-              session_clean (client_session, client_credentials);
+              session_clean (client_connection);
               return 0;
             }
           else if (ret == -10)
             {
               /* Now in a process forked to run a task, which has
                * failed in starting the task. */
-              session_clean (client_session, client_credentials);
+              session_clean (client_connection);
               return -1;
             }
           else if (ret == -1 || ret == -4)
@@ -641,25 +753,23 @@ serve_omp (gnutls_session_t* client_session,
               /* Error.  Write rest of to_client to client, so that the
                * client gets any buffered output and the response to the
                * error. */
-              write_to_client (client_session);
+              write_to_client (client_connection);
               rc = -1;
               goto client_free;
             }
           else if (ret == -2)
             {
               /* to_scanner buffer full. */
-              tracef ("   client input stalled 1\n");
+              g_debug ("   client input stalled 1\n");
               client_input_stalled = 1;
-              /* Break to write to_scanner. */
-              break;
+              /* Carry on to write to_scanner. */
             }
           else if (ret == -3)
             {
               /* to_client buffer full. */
-              tracef ("   client input stalled 2\n");
+              g_debug ("   client input stalled 2\n");
               client_input_stalled = 2;
-              /* Break to write to_client. */
-              break;
+              /* Carry on to write to_client. */
             }
           else
             {
@@ -725,11 +835,12 @@ serve_omp (gnutls_session_t* client_session,
         }
 
       /* Write any data to the client. */
-      if (client_socket > 0 && FD_ISSET (client_socket, &writefds))
+      if (client_connection->socket > 0
+          && FD_ISSET (client_connection->socket, &writefds))
         {
           /* Write as much as possible to the client. */
 
-          switch (write_to_client (client_session))
+          switch (write_to_client (client_connection))
             {
               case  0:      /* Wrote everything in to_client. */
                 break;
@@ -758,7 +869,7 @@ serve_omp (gnutls_session_t* client_session,
                * without closing it, for usage by the child process. */
               openvas_scanner_free ();
               set_scanner_init_state (SCANNER_INIT_TOP);
-              nfds = openvas_scanner_get_nfds (client_socket);
+              nfds = openvas_scanner_get_nfds (client_connection->socket);
               /* Skip the rest of the loop because the scanner socket is
                * a new socket.  This is asking for select trouble, really. */
               continue;
@@ -769,7 +880,7 @@ serve_omp (gnutls_session_t* client_session,
                * successfully started the task.  Close the client
                * connection, as the parent process has continued the
                * session with the client. */
-              session_clean (client_session, client_credentials);
+              session_clean (client_connection);
               scan_handler = 1;
               client_active = 0;
             }
@@ -779,14 +890,14 @@ serve_omp (gnutls_session_t* client_session,
                * successfully completed.  Close the client connection,
                * and exit, as the parent process has continued the
                * session with the client. */
-              session_clean (client_session, client_credentials);
+              session_clean (client_connection);
               return 0;
             }
           else if (ret == -10)
             {
               /* Now in a process forked to run a task, which has
                * failed in starting the task. */
-              session_clean (client_session, client_credentials);
+              session_clean (client_connection);
               return -1;
             }
           else if (ret == -1)
@@ -794,20 +905,20 @@ serve_omp (gnutls_session_t* client_session,
               /* Error.  Write rest of to_client to client, so that the
                * client gets any buffered output and the response to the
                * error. */
-              write_to_client (client_session);
+              write_to_client (client_connection);
               rc = -1;
               goto client_free;
             }
           else if (ret == -2)
             {
               /* to_scanner buffer full. */
-              tracef ("   client input still stalled (1)\n");
+              g_debug ("   client input still stalled (1)\n");
               client_input_stalled = 1;
             }
           else if (ret == -3)
             {
               /* to_client buffer full. */
-              tracef ("   client input still stalled (2)\n");
+              g_debug ("   client input still stalled (2)\n");
               client_input_stalled = 2;
             }
           else
@@ -834,7 +945,7 @@ serve_omp (gnutls_session_t* client_session,
               if (client_active == 0)
                 return 0;
               openvas_scanner_free ();
-              nfds = openvas_scanner_get_nfds (client_socket);
+              nfds = openvas_scanner_get_nfds (client_connection->socket);
             }
           else if (ret == 2)
             {
@@ -859,7 +970,7 @@ serve_omp (gnutls_session_t* client_session,
             }
           else if (ret == -3)
             /* to_scanner buffer still full. */
-            tracef ("   scanner input stalled\n");
+            g_debug ("   scanner input stalled\n");
           else
             /* Programming error. */
             assert (ret == 0);
@@ -875,8 +986,6 @@ serve_omp (gnutls_session_t* client_session,
 
 client_free:
   if (client_active)
-    openvas_server_free (client_socket,
-                         *client_session,
-                         *client_credentials);
+    openvas_connection_free (client_connection);
   return rc;
 }
