@@ -27,6 +27,7 @@
 #define OPENVAS_MANAGER_OMPD_H
 
 #include "types.h"
+#include <openvas/misc/openvas_server.h>
 #include <glib.h>
 #include <netinet/in.h>
 #include <gnutls/gnutls.h>
@@ -52,19 +53,18 @@ extern char *from_scanner;
 extern buffer_size_t from_scanner_start;
 extern buffer_size_t from_scanner_end;
 
-/*@-exportlocal@*/
 extern struct sockaddr_in scanner_address;
-/*@=exportlocal@*/
 
 int
-init_ompd (GSList*, int, const gchar*, int, int, int, void (*) (), int);
+init_ompd (GSList*, int, const gchar*, int, int, int, void (*) (),
+           int (*) (openvas_connection_t *, gchar *),
+           int);
 
 void
 init_ompd_process (const gchar *, gchar **);
 
 int
-serve_omp (gnutls_session_t*, gnutls_certificate_credentials_t*,
-           int, const gchar*, gchar**, void (*progress) ());
+serve_omp (openvas_connection_t*, const gchar*, gchar**, void (*progress) ());
 
 /** @todo Temporarily declared here, for omp.c SEND_TO_CLIENT. */
 extern char to_client[];
