@@ -45,8 +45,6 @@
 #include "otp.h"
 #include "manage.h"
 #include "scanner.h"
-#include "manage_sql.h"
-#include "tracef.h"
 #include "types.h"
 
 #include <assert.h>
@@ -60,10 +58,6 @@
 #include <openvas/base/openvas_string.h>
 #include <openvas/misc/nvt_categories.h>
 
-#ifdef S_SPLINT_S
-#include "splint.h"
-#endif
-
 #undef G_LOG_DOMAIN
 /**
  * @brief GLib log domain.
@@ -112,7 +106,6 @@ blank_control_chars (char *string)
 /**
  * @brief Current message during OTP SERVER message commands.
  */
-/*@null@*/ /*@only@*/
 static message_t* current_message = NULL;
 
 /**
@@ -129,10 +122,8 @@ static char *plugins_feed_version = NULL;
  *
  * @return A pointer to the new message.
  */
-/*@only@*/
 static message_t*
 make_message (const char* host)
-  /*@ensures isnull result->description, result->oid@*/
 {
   message_t* message;
 
@@ -154,7 +145,7 @@ make_message (const char* host)
  * @param[in]  message       Pointer to the message.
  */
 static void
-free_message (/*@out@*/ /*@only@*/ message_t* message)
+free_message (message_t* message)
 {
   if (message->host) free (message->host);
   if (message->description) free (message->description);
@@ -215,7 +206,7 @@ set_message_port_string (message_t* message, char* string)
  * @param[in]  description  Description.
  */
 static void
-set_message_description (message_t* message, /*@only@*/ char* description)
+set_message_description (message_t* message, char* description)
 {
   if (message->description) free (message->description);
   message->description = description;
@@ -229,7 +220,7 @@ set_message_description (message_t* message, /*@only@*/ char* description)
  * @param[in]  oid          OID.
  */
 static void
-set_message_oid (message_t* message, /*@only@*/ char* oid)
+set_message_oid (message_t* message, char* oid)
 {
   if (message->oid) free (message->oid);
   message->oid = oid;
@@ -319,7 +310,6 @@ append_log_message (task_t task, message_t* message)
 /**
  * @brief The current scanner preference, during reading of scanner preferences.
  */
-/*@null@*/ /*@only@*/
 static char* current_scanner_preference = NULL;
 
 
@@ -328,7 +318,6 @@ static char* current_scanner_preference = NULL;
 /**
  * @brief The current plugin, during reading of scanner plugin list.
  */
-/*@only@*/
 static nvti_t* current_plugin = NULL;
 
 /**
@@ -336,6 +325,11 @@ static nvti_t* current_plugin = NULL;
  */
 GList* scanner_plugins_list = NULL;
 
+/**
+ * @brief The full preferences list, during reading of scanner plugin list.
+ */
+GList* scanner_preferences_list = NULL;
+
 
 /* Scanner state. */
 
@@ -378,7 +372,6 @@ typedef enum
   SCANNER_PLUGIN_LIST_NAME,
   SCANNER_PLUGIN_LIST_OID,
   SCANNER_PLUGIN_LIST_PLUGIN_VERSION,
-  SCANNER_PLUGIN_LIST_SUMMARY,
   SCANNER_PLUGIN_LIST_TAGS,
   SCANNER_PLUGIN_LIST_XREFS,
   SCANNER_PREFERENCE_NAME,
@@ -409,7 +402,7 @@ static void
 set_scanner_state (scanner_state_t state)
 {
   scanner_state = state;
-  tracef ("   scanner state set: %i\n", scanner_state);
+  g_debug ("   scanner state set: %i\n", scanner_state);
 }
 
 /**
@@ -439,7 +432,20 @@ void
 set_scanner_init_state (scanner_init_state_t state)
 {
   scanner_init_state = state;
-  tracef ("   scanner init state set: %i\n", scanner_init_state);
+  g_debug ("   scanner init state set: %i\n", scanner_init_state);
+}
+
+/**
+ * @brief Set the scanner initialisation state, \ref scanner_init_state.
+ */
+void
+reset_scanner_states ()
+{
+  set_scanner_state (SCANNER_TOP);
+  set_scanner_init_state (SCANNER_INIT_TOP);
+  scanner_init_offset = 0;
+  scanner_current_loading = 0;
+  scanner_total_loading = 0;
 }
 
 
@@ -464,7 +470,7 @@ sync_buffer ()
   if (from_scanner_start > 0 && from_scanner_start == from_scanner_end)
     {
       from_scanner_start = from_scanner_end = 0;
-      tracef ("   scanner start caught end\n");
+      g_debug ("   scanner start caught end\n");
     }
   else if (from_scanner_start == 0)
     {
@@ -483,14 +489,11 @@ sync_buffer ()
       from_scanner_end -= from_scanner_start;
       memmove (from_scanner, start, from_scanner_end);
       from_scanner_start = 0;
-#if TRACE
       from_scanner[from_scanner_end] = '\0';
-      //tracef ("   new from_scanner: %s\n", from_scanner);
-      tracef ("   new from_scanner_start: %" BUFFER_SIZE_T_FORMAT "\n",
-              from_scanner_start);
-      tracef ("   new from_scanner_end: %" BUFFER_SIZE_T_FORMAT "\n",
-              from_scanner_end);
-#endif
+      g_debug ("   new from_scanner_start: %" BUFFER_SIZE_T_FORMAT "\n",
+               from_scanner_start);
+      g_debug ("   new from_scanner_end: %" BUFFER_SIZE_T_FORMAT "\n",
+               from_scanner_end);
     }
   return 0;
 }
@@ -514,7 +517,7 @@ parse_scanner_done (char** messages)
     return -2;
   if (strncasecmp ("SERVER", *messages, 6))
     {
-      tracef ("   scanner fail: expected final \"SERVER\"\n");
+      g_debug ("   scanner fail: expected final \"SERVER\"\n");
       return -1;
     }
   set_scanner_state (SCANNER_TOP);
@@ -533,7 +536,7 @@ parse_scanner_done (char** messages)
 static int
 parse_scanner_bad_login (char** messages)
 {
-  /*@dependent@*/ char *end, *match;
+  char *end, *match;
   end = *messages + from_scanner_end - from_scanner_start;
   while (*messages < end && ((*messages)[0] == ' '))
     { (*messages)++; from_scanner_start++; }
@@ -544,7 +547,7 @@ parse_scanner_bad_login (char** messages)
       /** @todo Are there 19 characters available? */
       if (strncasecmp ("Bad login attempt !", *messages, 19) == 0)
         {
-          tracef ("match bad login\n");
+          g_debug ("match bad login\n");
           from_scanner_start += match + 1 - *messages;
           *messages = match + 1;
           set_scanner_init_state (SCANNER_INIT_TOP);
@@ -574,16 +577,21 @@ parse_scanner_preference_value (char** messages, void (*progress) ())
                        from_scanner_end - from_scanner_start)))
     {
       match[0] = '\0';
-      value = g_strdup (*messages);
-      blank_control_chars (value);
       if (current_scanner_preference)
         {
+          preference_t *preference;
+
+          value = g_strdup (*messages);
+          blank_control_chars (value);
           if (progress)
             progress ();
-          if (scanner_init_state == SCANNER_INIT_DONE_CACHE_MODE)
-            manage_nvt_preference_add (current_scanner_preference, value, 0);
-          else if (scanner_init_state == SCANNER_INIT_DONE_CACHE_MODE_UPDATE)
-            manage_nvt_preference_add (current_scanner_preference, value, 1);
+          preference = g_malloc0 (sizeof (preference_t));
+          preference->name = g_strdup (current_scanner_preference);
+          preference->value = value;
+          /* Add the preference to the_list which will be bulk-inserted
+           * in DB later in manage_complete_nvt_cache_update. */
+          scanner_preferences_list = g_list_prepend (scanner_preferences_list,
+                                                     preference);
         }
       set_scanner_state (SCANNER_PREFERENCE_NAME);
       from_scanner_start += match + 1 - *messages;
@@ -627,13 +635,12 @@ parse_scanner_plugin_list_tags (char** messages)
         }
       if (current_plugin)
         {
-          gchar *tags, *cvss_base, *risk_factor;
-          parse_tags (value, &tags, &cvss_base, &risk_factor);
+          gchar *tags, *cvss_base;
+          parse_tags (value, &tags, &cvss_base);
           nvti_set_tag (current_plugin, tags);
           nvti_set_cvss_base (current_plugin, cvss_base);
           g_free (tags);
           g_free (cvss_base);
-          g_free (risk_factor);
 
           /* Add the plugin to scanner_plugins_list which will be bulk-inserted
            * in DB later in manage_complete_nvt_cache_update. */
@@ -660,9 +667,9 @@ parse_scanner_plugin_list_tags (char** messages)
  *         field follows), -4 failed to find a newline (may be a <|>)
  */
 static int
-parse_scanner_server (/*@dependent@*/ char** messages)
+parse_scanner_server (char** messages)
 {
-  /*@dependent@*/ char *end, *match;
+  char *end, *match;
   end = *messages + from_scanner_end - from_scanner_start;
   while (*messages < end && ((*messages)[0] == ' '))
     { (*messages)++; from_scanner_start++; }
@@ -670,8 +677,8 @@ parse_scanner_server (/*@dependent@*/ char** messages)
                        (int) '\n',
                        from_scanner_end - from_scanner_start)))
     {
-      /*@dependent@*/ char* newline;
-      /*@dependent@*/ char* input;
+      char* newline;
+      char* input;
       buffer_size_t from_start, from_end;
       match[0] = '\0';
       /** @todo Is there ever whitespace before the newline? */
@@ -755,7 +762,8 @@ parse_scanner_loading (char *messages)
  *
  * This includes updating the scanner state with \ref set_scanner_state
  * and \ref set_scanner_init_state, and updating scanner records with functions
- * like \ref manage_nvt_preference_add.
+ * like \ref manage_nvt_preference_add (via
+ * \ref manage_complete_nvt_cache_update).
  *
  * \endif
  *
@@ -772,13 +780,13 @@ parse_scanner_loading (char *messages)
 int
 process_otp_scanner_input (void (*progress) ())
 {
-  /*@dependent@*/ char* match = NULL;
-  /*@dependent@*/ char* messages = from_scanner + from_scanner_start;
-  /*@dependent@*/ char* input;
+  char* match = NULL;
+  char* messages = from_scanner + from_scanner_start;
+  char* input;
   const char *ver_str = "< OTP/2.0 >\n";
   size_t ver_len = strlen (ver_str);
   buffer_size_t from_start, from_end;
-  //tracef ("   consider %.*s\n", from_scanner_end - from_scanner_start, messages);
+  //g_debug ("   consider %.*s\n", from_scanner_end - from_scanner_start, messages);
 
   /* Before processing the input, check if another manager process has stopped
    * the current task.  If so, send the stop request to the scanner.  This is
@@ -808,11 +816,11 @@ process_otp_scanner_input (void (*progress) ())
             parse_scanner_loading (messages);
             if (scanner_current_loading && scanner_total_loading)
               g_log (G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
-                     "Scanner loading: %d / %d nvts.\n",
+                     "Waiting for scanner to load NVTs: %d / %d.\n",
                      scanner_current_loading, scanner_total_loading);
             else
               g_log (G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
-                     "Scanner loading: No information provided. %s\n", messages);
+                     "Waiting for scanner to load: No information provided. (Message: %s)\n", messages);
             return 3;
           }
         if (from_scanner_end - from_scanner_start < ver_len)
@@ -823,8 +831,8 @@ process_otp_scanner_input (void (*progress) ())
           }
         if (strncasecmp (ver_str, messages, ver_len))
           {
-            tracef ("   scanner fail: expected \"%s\""
-                    "   got \"%.12s\"\n\n", ver_str, messages);
+            g_debug ("   scanner fail: expected \"%s\""
+                     "   got \"%.12s\"\n\n", ver_str, messages);
             return -1;
           }
         from_scanner_start += ver_len;
@@ -935,8 +943,8 @@ process_otp_scanner_input (void (*progress) ())
           field = openvas_strip_space (message, match);
           blank_control_chars (field);
 
-          tracef ("   scanner old state %i\n", scanner_state);
-          tracef ("   scanner field: %s\n", field);
+          g_debug ("   scanner old state %i\n", scanner_state);
+          g_debug ("   scanner field: %s\n", field);
           switch (scanner_state)
             {
               case SCANNER_BYE:
@@ -986,7 +994,6 @@ process_otp_scanner_input (void (*progress) ())
 
                   protocol = g_newa (char, strlen (field));
 
-                  /* RATS: ignore, buffers are allocated to field length. */
                   if (sscanf (field, "%i/%s",
                               &number, protocol)
                       != 2)
@@ -994,8 +1001,8 @@ process_otp_scanner_input (void (*progress) ())
                       number = atoi (field);
                       protocol[0] = '\0';
                     }
-                  tracef ("   scanner got debug port, number: %i, protocol: %s\n",
-                          number, protocol);
+                  g_debug ("   scanner got debug port, number: %i, protocol: %s\n",
+                           number, protocol);
 
                   set_message_port_number (current_message, number);
                   set_message_port_protocol (current_message, protocol);
@@ -1059,7 +1066,6 @@ process_otp_scanner_input (void (*progress) ())
 
                   protocol = g_newa (char, strlen (field));
 
-                  /* RATS: ignore, buffers are allocated to field length. */
                   if (sscanf (field, "%i/%s",
                               &number, protocol)
                       != 2)
@@ -1067,8 +1073,8 @@ process_otp_scanner_input (void (*progress) ())
                       number = atoi (field);
                       protocol[0] = '\0';
                     }
-                  tracef ("   scanner got alarm port, number: %i, protocol: %s\n",
-                          number, protocol);
+                  g_debug ("   scanner got alarm port, number: %i, protocol: %s\n",
+                           number, protocol);
 
                   set_message_port_number (current_message, number);
                   set_message_port_protocol (current_message, protocol);
@@ -1132,7 +1138,6 @@ process_otp_scanner_input (void (*progress) ())
 
                   protocol = g_newa (char, strlen (field));
 
-                  /* RATS: ignore, buffers are allocated to field length. */
                   if (sscanf (field, "%i/%s",
                               &number, protocol)
                       != 2)
@@ -1140,8 +1145,8 @@ process_otp_scanner_input (void (*progress) ())
                       number = atoi (field);
                       protocol[0] = '\0';
                     }
-                  tracef ("   scanner got log port, number: %i, protocol: %s\n",
-                          number, protocol);
+                  g_debug ("   scanner got log port, number: %i, protocol: %s\n",
+                           number, protocol);
 
                   set_message_port_number (current_message, number);
                   set_message_port_protocol (current_message, protocol);
@@ -1227,12 +1232,6 @@ process_otp_scanner_input (void (*progress) ())
               case SCANNER_PLUGIN_LIST_COPYRIGHT:
                 {
                   nvti_set_copyright (current_plugin, field);
-                  set_scanner_state (SCANNER_PLUGIN_LIST_SUMMARY);
-                  break;
-                }
-              case SCANNER_PLUGIN_LIST_SUMMARY:
-                {
-                  nvti_set_summary (current_plugin, field);
                   set_scanner_state (SCANNER_PLUGIN_LIST_FAMILY);
                   break;
                 }
@@ -1276,7 +1275,7 @@ process_otp_scanner_input (void (*progress) ())
               case SCANNER_NVT_INFO:
                 {
                   char* feed_version = g_strdup (field);
-                  tracef ("   scanner got nvti_info: %s\n", feed_version);
+                  g_debug ("   scanner got nvti_info: %s\n", feed_version);
                   if (plugins_feed_version)
                     g_free (plugins_feed_version);
                   plugins_feed_version = feed_version;
@@ -1317,6 +1316,7 @@ process_otp_scanner_input (void (*progress) ())
                         {
                           manage_complete_nvt_cache_update
                            (scanner_plugins_list,
+                            scanner_preferences_list,
                             scanner_init_state == SCANNER_INIT_DONE_CACHE_MODE
                             ? -2 : -1);
                           set_scanner_init_state (SCANNER_INIT_DONE);
@@ -1396,8 +1396,8 @@ process_otp_scanner_input (void (*progress) ())
                   }
                 else
                   {
-                    tracef ("New scanner command to implement: %s\n",
-                            field);
+                    g_debug ("New scanner command to implement: %s\n",
+                             field);
                     goto return_error;
                   }
                 break;
@@ -1415,7 +1415,7 @@ process_otp_scanner_input (void (*progress) ())
                   if (current_report && current_host)
                     {
                       unsigned int current, max;
-                      tracef ("   scanner got ports: %s\n", field);
+                      g_debug ("   scanner got ports: %s\n", field);
                       if (sscanf (field, "%u/%u", &current, &max) == 2)
                         set_scan_ports (current_report,
                                         current_host,
@@ -1465,6 +1465,8 @@ process_otp_scanner_input (void (*progress) ())
                   if (current_scanner_task)
                     {
                       assert (current_host);
+                      assert (current_report);
+
                       set_scan_host_start_time_otp (current_report,
                                                     current_host,
                                                     field);
@@ -1491,6 +1493,18 @@ process_otp_scanner_input (void (*progress) ())
                 }
               case SCANNER_TIME_HOST_END_TIME:
                 {
+                  assert (current_host);
+                  assert (current_report);
+
+                  if (report_host_noticeable (current_report, current_host))
+                    {
+                      char *uuid;
+                      uuid = report_uuid (current_report);
+                      host_notice (current_host, "ip", current_host,
+                                   "Report Host", uuid, 1, 0);
+                      free (uuid);
+                    }
+
                   if (current_scanner_task)
                     {
                       assert (current_host);
@@ -1548,8 +1562,13 @@ process_otp_scanner_input (void (*progress) ())
                       /* Stop transaction now, because delete_task_lock and
                        * set_scan_end_time_otp run transactions themselves. */
                       manage_transaction_stop (TRUE);
+                      hosts_set_identifiers ();
                       if (current_report)
-                        set_scan_end_time_otp (current_report, field);
+                        {
+                          hosts_set_max_severity (current_report, NULL, NULL);
+                          hosts_set_details (current_report);
+                          set_scan_end_time_otp (current_report, field);
+                        }
                       switch (task_run_status (current_scanner_task))
                         {
                           case TASK_STATUS_INTERNAL_ERROR:
@@ -1597,8 +1616,8 @@ process_otp_scanner_input (void (*progress) ())
                 }
               case SCANNER_TOP:
               default:
-                tracef ("   switch t\n");
-                tracef ("   cmp %i\n", strcasecmp ("SERVER", field));
+                g_debug ("   switch t\n");
+                g_debug ("   cmp %i\n", strcasecmp ("SERVER", field));
                 if (strcasecmp ("SERVER", field))
                   goto return_error;
                 set_scanner_state (SCANNER_SERVER);
@@ -1617,7 +1636,7 @@ process_otp_scanner_input (void (*progress) ())
                 break;
             }
 
-          tracef ("   scanner new state: %i\n", scanner_state);
+          g_debug ("   scanner new state: %i\n", scanner_state);
 
           continue;
 
