@@ -59,10 +59,14 @@ extern scanner_init_state_t scanner_init_state;
 extern int scanner_current_loading;
 extern int scanner_total_loading;
 
-/** @todo Exported for ompd.c. */
+/** @todo Exported for ompd.c and scanner.c. */
 void
 set_scanner_init_state (scanner_init_state_t state);
 
+/** @todo Exported for scanner.c. */
+void
+reset_scanner_states ();
+
 /** @todo Exported for ompd.c. */
 extern int scanner_init_offset;
 #endif
