@@ -24,12 +24,17 @@
  */
 
 #include "sql.h"
-#include "tracef.h"
 
 #include <assert.h>
 #include <errno.h>
+#include <stdlib.h>
 #include <string.h>
-#include <time.h>
+
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md manage"
 
 
 /* Headers of internal symbols defined in backend files. */
@@ -49,50 +54,6 @@ sql_explain (const char*, ...);
 
 /* Helpers. */
 
-/**
- * @brief Sleep for some number of microseconds, handling interrupts.
- *
- * @param[in] microseconds  Number of microseconds.
- *
- * @return 0 success, -1 error (with errno set).
- */
-int
-openvas_usleep (unsigned int microseconds)
-{
-  struct timespec a, b, *requested, *remaining;
-  int ret;
-
-  requested = &a;
-  remaining = &b;
-
-  requested->tv_sec = microseconds / 1000000;
-  requested->tv_nsec = (microseconds % 1000000) * 1000;
-
-  while ((ret = nanosleep (requested, remaining)) && (errno == EINTR))
-    {
-      struct timespec *temp;
-      temp = requested;
-      requested = remaining;
-      remaining = temp;
-    }
-  if (ret)
-    return -1;
-  return 0;
-}
-
-/**
- * @brief Sleep for some number of seconds, handling interrupts.
- *
- * @param[in] seconds  Number of seconds.
- *
- * @return 0 success, -1 error (with errno set).
- */
-int
-openvas_sleep (unsigned int seconds)
-{
-  return openvas_usleep (seconds * 1000000);
-}
-
 /**
  * @brief Quotes a string of a known length to be passed to sql statements.
  *
@@ -342,7 +303,7 @@ sql_quiet (char* sql, ...)
 /**
  * @brief Get a particular cell from a SQL query.
  *
- * @param[in]   log          Whether to do tracef logging.
+ * @param[in]   log          Whether to do g_debug logging.
  * @param[in]   sql          Format string for SQL query.
  * @param[in]   args         Arguments for format string.
  * @param[out]  stmt_return  Return from statement.
@@ -393,7 +354,7 @@ sql_x_internal (int log, char* sql, va_list args, sql_stmt_t** stmt_return)
     }
   assert (ret == 1);
   if (log)
-    tracef ("   sql_x end\n");
+    g_debug ("   sql_x end (%s)\n", sql);
   return 0;
 }
 
@@ -591,7 +552,7 @@ init_prepared_iterator (iterator_t* iterator, sql_stmt_t* stmt)
   iterator->stmt = stmt;
   iterator->prepared = 1;
   iterator->crypt_ctx = NULL;
-  tracef ("   sql: init prepared %p\n", stmt);
+  g_debug ("   sql: init prepared %p\n", stmt);
 }
 
 /**
