@@ -24,19 +24,26 @@
  */
 
 #include "sql.h"
-#include "tracef.h"
 
 #include <assert.h>
 #include <endian.h>
+#include <errno.h>
 #include <arpa/inet.h>
 #include <glib.h>
 #include <inttypes.h>
 #include <netinet/in.h>
 #include <postgresql/libpq-fe.h>
+#include <stdlib.h>
 #include <string.h>
 
 #include <openvas/base/array.h>
 
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md manage"
+
 
 /* Headers of sql.c symbols used only here. */
 
@@ -90,6 +97,17 @@ sql_schema ()
   return "public";
 }
 
+/**
+ * @brief Get keyword for "greatest" SQL function.
+ *
+ * @return Keyword.
+ */
+const char *
+sql_greatest ()
+{
+  return "greatest";
+}
+
 /**
  * @brief Setup a LIMIT argument.
  *
@@ -144,6 +162,17 @@ sql_stmt_init (sql_stmt_t *stmt)
   stmt->current_row = -1;
 }
 
+/**
+ * @brief Get case insensitive LIKE operator.
+ *
+ * @return Like operator.
+ */
+const char *
+sql_ilike_op ()
+{
+  return "ILIKE";
+}
+
 /**
  * @brief Get regular expression operator.
  *
@@ -213,29 +242,109 @@ int
 sql_open (const char *database)
 {
   gchar *conn_info;
+  PostgresPollingStatusType poll_status;
+  int socket;
 
   conn_info = g_strdup_printf ("dbname = %s",
                                database
                                 ? database
                                 : sql_default_database ());
-  conn = PQconnectdb (conn_info);
+  conn = PQconnectStart (conn_info);
   g_free (conn_info);
-  if (PQstatus (conn) != CONNECTION_OK)
+  if (conn == NULL)
     {
-      g_warning ("%s: PQconnectdb to '%s' failed: %s\n",
+      g_warning ("%s: PQconnectStart failed to allocate conn\n",
+                 __FUNCTION__);
+      return -1;
+    }
+  if (PQstatus (conn) == CONNECTION_BAD)
+    {
+      g_warning ("%s: PQconnectStart to '%s' failed: %s\n",
                  __FUNCTION__,
                  database ? database : sql_default_database (),
                  PQerrorMessage (conn));
-      return -1;
+      goto fail;
+    }
+
+  socket = PQsocket (conn);
+  if (socket == 0)
+    {
+      g_warning ("%s: PQsocket 0\n", __FUNCTION__);
+      goto fail;
+    }
+
+  poll_status = PGRES_POLLING_WRITING;
+
+  g_debug ("%s: polling\n", __FUNCTION__);
+
+  while (1)
+    {
+      if (poll_status == PGRES_POLLING_READING)
+        {
+          fd_set readfds, writefds;
+          int ret;
+
+          FD_ZERO (&readfds);
+          FD_ZERO (&writefds);
+          FD_SET (socket, &readfds);
+          ret = select (socket + 1, &readfds, &writefds, NULL, NULL);
+          if (ret == 0)
+            continue;
+          if (ret < 0)
+            {
+              g_warning ("%s: write select failed: %s\n",
+                         __FUNCTION__, strerror (errno));
+              goto fail;
+            }
+          /* Poll again. */
+        }
+      else if (poll_status == PGRES_POLLING_WRITING)
+        {
+          fd_set readfds, writefds;
+          int ret;
+
+          FD_ZERO (&readfds);
+          FD_ZERO (&writefds);
+          FD_SET (socket, &writefds);
+          ret = select (socket + 1, &readfds, &writefds, NULL, NULL);
+          if (ret == 0)
+            continue;
+          if (ret < 0)
+            {
+              g_warning ("%s: read select failed: %s\n",
+                         __FUNCTION__, strerror (errno));
+              goto fail;
+            }
+          /* Poll again. */
+        }
+      else if (poll_status == PGRES_POLLING_FAILED)
+        {
+          g_warning ("%s: PQconnectPoll failed\n",
+                     __FUNCTION__);
+          goto fail;
+        }
+      else if (poll_status == PGRES_POLLING_OK)
+        /* Connection is ready, exit loop. */
+        break;
+
+      poll_status = PQconnectPoll (conn);
     }
 
   PQsetNoticeProcessor (conn, log_notice, NULL);
 
-  tracef ("%s:   db: %s\n", __FUNCTION__, PQdb (conn));
-  tracef ("%s: user: %s\n", __FUNCTION__, PQuser (conn));
-  tracef ("%s: port: %s\n", __FUNCTION__, PQport (conn));
+  g_debug ("%s:   db: %s\n", __FUNCTION__, PQdb (conn));
+  g_debug ("%s: user: %s\n", __FUNCTION__, PQuser (conn));
+  g_debug ("%s: host: %s\n", __FUNCTION__, PQhost (conn));
+  g_debug ("%s: port: %s\n", __FUNCTION__, PQport (conn));
+  g_debug ("%s: socket: %i\n", __FUNCTION__, PQsocket (conn));
+  g_debug ("%s: postgres version: %i\n", __FUNCTION__, PQserverVersion (conn));
 
   return 0;
+
+ fail:
+  PQfinish (conn);
+  conn = NULL;
+  return -1;
 }
 
 /**
@@ -347,7 +456,7 @@ sql_prepare_internal (int retry, int log, const char* sql, va_list args,
   (*stmt)->sql = g_strdup_vprintf (sql, args);
 
   if (log)
-    tracef ("   sql: %s\n", (*stmt)->sql);
+    g_debug ("   sql: %s\n", (*stmt)->sql);
 
   return 0;
 }
@@ -466,7 +575,7 @@ void
 sql_begin_exclusive ()
 {
   sql ("BEGIN;");
-  sql ("LOCK TABLE meta IN ACCESS EXCLUSIVE MODE;");
+  sql ("SELECT pg_advisory_xact_lock (1);");
 }
 
 /**
@@ -482,13 +591,10 @@ sql_begin_exclusive_giveup ()
   ret = sql_giveup ("BEGIN;");
   if (ret)
     return ret;
-  ret = sql_giveup ("LOCK TABLE meta IN ACCESS EXCLUSIVE MODE;");
-  if (ret)
-    {
-      sql ("ROLLBACK;");
-      return ret;
-    }
-  return 0;
+  if (sql_int ("SELECT pg_try_advisory_xact_lock (1);"))
+    return 0;
+  sql_rollback ();
+  return 1;
 }
 
 /**
@@ -497,9 +603,8 @@ sql_begin_exclusive_giveup ()
 void
 sql_begin_immediate ()
 {
-  sql ("BEGIN;");
   /* TODO This is just an exclusive lock. */
-  sql ("LOCK TABLE meta IN ACCESS EXCLUSIVE MODE;");
+  sql_begin_exclusive ();
 }
 
 /**
@@ -514,6 +619,24 @@ sql_begin_immediate_giveup ()
   return sql_begin_exclusive_giveup ();
 }
 
+/**
+ * @brief Commit a transaction.
+ */
+void
+sql_commit ()
+{
+  sql ("COMMIT;");
+}
+
+/**
+ * @brief Roll a transaction back.
+ */
+void
+sql_rollback ()
+{
+  sql ("ROLLBACK;");
+}
+
 
 /* Iterators. */
 
@@ -643,6 +766,7 @@ sql_bind_double (sql_stmt_t *stmt, int position, double *value)
   gchar *string;
   string = g_strdup_printf ("%f", *value);
   bind_param (stmt, position, string, strlen (string), 0);
+  g_free (string);
   return 0;
 }
 
