@@ -24,17 +24,27 @@
  */
 
 #include "sql.h"
-#include "tracef.h"
+#include "utils.h"
 
 #include <assert.h>
 #include <sqlite3.h>
+#include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <errno.h>
 
 /**
  * @brief Chunk size for SQLite memory allocation.
  */
 #define DB_CHUNK_SIZE 1 * 1024 * 1024
 
+#undef G_LOG_DOMAIN
+/**
+ * @brief GLib log domain.
+ */
+#define G_LOG_DOMAIN "md manage"
+
 
 /* Headers of sql.c symbols used only here. */
 
@@ -82,6 +92,17 @@ sql_schema ()
   return "main";
 }
 
+/**
+ * @brief Get keyword for "greatest" SQL function.
+ *
+ * @return Keyword.
+ */
+const char *
+sql_greatest ()
+{
+  return "max";
+}
+
 /**
  * @brief Setup a LIMIT argument.
  *
@@ -104,6 +125,17 @@ sql_select_limit (int max)
   return string;
 }
 
+/**
+ * @brief Get case insensitive LIKE operator.
+ *
+ * @return Like operator.
+ */
+const char *
+sql_ilike_op ()
+{
+  return "LIKE";
+}
+
 /**
  * @brief Get regular expression operator.
  *
@@ -160,12 +192,55 @@ int
 sql_open (const char *database)
 {
   int chunk_size = DB_CHUNK_SIZE;
+  struct stat state;
+  int err, ret;
+  gchar *mgr_dir;
 
-  // Workaround for SQLite temp file name conflicts that can occur if
-  //  concurrent forked processes have the same PRNG state.
-  #if SQLITE_VERSION_NUMBER < 3008003
+  /* Ensure the mgr directory exists. */
+
+  mgr_dir = g_build_filename (OPENVAS_STATE_DIR, "mgr", NULL);
+  ret = g_mkdir_with_parents (mgr_dir, 0755 /* "rwxr-xr-x" */);
+  g_free (mgr_dir);
+  if (ret == -1)
+    {
+      g_warning ("%s: failed to create mgr directory: %s\n",
+                 __FUNCTION__,
+                 strerror (errno));
+      abort ();
+    }
+
+  err = stat (database ? database : sql_default_database (),
+              &state);
+  if (err)
+    switch (errno)
+      {
+        case ENOENT:
+          break;
+        default:
+          g_warning ("%s: failed to stat database: %s\n",
+                     __FUNCTION__,
+                     strerror (errno));
+          abort ();
+      }
+  else if (state.st_mode & (S_IXUSR | S_IRWXG | S_IRWXO))
+    {
+      g_warning ("%s: database permissions are too loose, repairing\n",
+                 __FUNCTION__);
+      if (chmod (database ? database : sql_default_database (),
+                 S_IRUSR | S_IWUSR))
+        {
+          g_warning ("%s: chmod failed: %s\n",
+                     __FUNCTION__,
+                     strerror (errno));
+          abort ();
+        }
+    }
+
+  /* Workaround for SQLite temp file name conflicts that can occur if
+   * concurrent forked processes have the same PRNG state. */
+#if SQLITE_VERSION_NUMBER < 3008003
     sqlite3_test_control (SQLITE_TESTCTRL_PRNG_RESET);
-  #endif
+#endif
 
   if (sqlite3_open (database ? database : sql_default_database (),
                     &task_db))
@@ -181,6 +256,10 @@ sql_open (const char *database)
            OPENVAS_SQLITE_SLEEP_MAX);
 
   sqlite3_file_control (task_db, NULL, SQLITE_FCNTL_CHUNK_SIZE, &chunk_size);
+
+  sql ("PRAGMA journal_mode=WAL;");
+  sql ("PRAGMA journal_size_limit=134217728;");  /* 128 MB. */
+
   return 0;
 }
 
@@ -276,7 +355,7 @@ sql_prepare_internal (int retry, int log, const char* sql, va_list args,
   formatted = g_strdup_vprintf (sql, args);
 
   if (log)
-    tracef ("   sql: %s\n", formatted);
+    g_debug ("   sql: %s\n", formatted);
 
   retries = 0;
   *stmt = (sql_stmt_t*) g_malloc0 (sizeof (sql_stmt_t));
@@ -459,6 +538,24 @@ sql_begin_immediate_giveup ()
   return sql_giveup ("BEGIN IMMEDIATE;");
 }
 
+/**
+ * @brief Commit a transaction.
+ */
+void
+sql_commit ()
+{
+  sql ("COMMIT;");
+}
+
+/**
+ * @brief Roll a transaction back.
+ */
+void
+sql_rollback ()
+{
+  sql ("ROLLBACK;");
+}
+
 
 /* Iterators. */
 
